#-*- coding: utf-8 -*-
# $Id: config.py 67354 2008-06-27 09:48:00Z glenfant $
"""Configuration data"""

from openxmllib import contenttypes as ct
import mimetypes

PROJECTNAME = 'OpenXml'
PROJECT_GLOBALS = globals()
I18N_DOMAIN = 'openxml' # FIXME: potentially useless for that kind of product
SITE_CHARSET = 'UTF-8'
TRANSFORM_NAME = 'openxml_to_words'

# New MIME types data for mimetypes_registry
# Source:
# * http://technet2.microsoft.com/Office/en-us/library/e077da98-0216-45eb-b6a7-957f9c510a851033.mspx
office_mimetypes = (
    {'name': 'Office Word 2007 XML macro enabled document',
     'mimetypes': (ct.CT_WORDPROC_DOCM_PUBLIC,),
     'extensions': ('docm',)
     },
    {'name': 'Office Word 2007 XML document',
     'mimetypes': (ct.CT_WORDPROC_DOCX_PUBLIC,),
     'extensions': ('docx',)
    },
    {'name': 'Office Word 2007 XML macro-enabled template',
     'mimetypes': (ct.CT_WORDPROC_DOTM_PUBLIC,),
     'extensions': ('dotm',)
    },
    {'name': 'Office Word 2007 XML template',
     'mimetypes': (ct.CT_WORDPROC_DOTX_PUBLIC,),
     'extensions': ('dotx',)
    },
    {'name': 'Office Powerpoint 2007 macro-enabled XML template',
     'mimetypes': (ct.CT_PRESENTATION_POTM_PUBLIC,),
     'extensions': ('potm',)
    },
    {'name': 'Office Powerpoint 2007 XML template',
     'mimetypes': (ct.CT_PRESENTATION_POTX_PUBLIC,),
     'extensions': ('potx',)
    },
    {'name': 'Office Powerpoint 2007 macro-enabled XML add-in',
     'mimetypes': (ct.CT_PRESENTATION_PPAM_PUBLIC,),
     'extensions': ('ppam',)
     },
    {'name': 'Office Powerpoint 2007 macro-enabled XML show',
     'mimetypes': (ct.CT_PRESENTATION_PPSM_PUBLIC,),
     'extensions': ('ppsm',)
     },
    {'name': 'Office Powerpoint 2007 XML show',
     'mimetypes': (ct.CT_PRESENTATION_PPSX_PUBLIC,),
     'extensions': ('ppsx',)
     },
    {'name': 'Office Powerpoint 2007 macro-enabled XML presentation',
     'mimetypes': (ct.CT_PRESENTATION_PPTM_PUBLIC,),
     'extensions': ('pptm',)
     },
    {'name': 'Office Powerpoint 2007 XML presentation',
     'mimetypes': (ct.CT_PRESENTATION_PPTX_PUBLIC,),
     'extensions': ('pptx',)
     },
    {'name': 'Office Excel 2007 XML macro-enabled add-in',
     'mimetypes': (ct.CT_SPREADSHEET_XLAM_PUBLIC,),
     'extensions': ('xlam',)
     },
    {'name': 'Office Excel 2007 binary workbook (BIFF12)',
     'mimetypes': (ct.CT_SPREADSHEET_XLSB_PUBLIC,),
     'extensions': ('xlsb',)
     },
    {'name': 'Office Excel 2007 XML macro-enabled workbook',
     'mimetypes': (ct.CT_SPREADSHEET_XLSM_PUBLIC,),
     'extensions': ('xlsm',)
     },
    {'name': 'Office Excel 2007 XML workbook',
     'mimetypes': (ct.CT_SPREADSHEET_XLSX_PUBLIC,),
     'extensions': ('xlsx',)
     },
    {'name': 'Office Excel 2007 XML macro-enabled template',
     'mimetypes': (ct.CT_SPREADSHEET_XLTM_PUBLIC,),
     'extensions': ('xltm',)
     },
    {'name': 'Office Excel 2007 XML template',
     'mimetypes': (ct.CT_SPREADSHEET_XLTX_PUBLIC,),
     'extensions': ('xltx',)
     },
    )

for mt in office_mimetypes:
    mt['globs'] = tuple(['*.' + ext for ext in mt['extensions']])
    mt['icon_path'] = '++resource++openxml-icons/%s.png' % mt['extensions'][0]
    # Adding to standard mimetypes
    mimetypes.add_type(mt['mimetypes'][0], '.' + mt['extensions'][0])

del ct
del mimetypes



#-*- coding: utf-8 -*-
# $Id: common.py 57531 2008-01-23 16:40:27Z glenfant $
"""OpenXml testing package: testing resources"""

from Products.PloneTestCase import PloneTestCase
from Products.OpenXml.config import PROJECTNAME

PloneTestCase.installProduct(PROJECTNAME)
PloneTestCase.setupPloneSite(products=[PROJECTNAME])




#-*- coding: utf-8 -*-
# $Id: test_mtr.py 57531 2008-01-23 16:40:27Z glenfant $
"""Testing mimetypes_registry settings"""
from Products.CMFCore.utils import getToolByName
from Products.MimetypesRegistry.interfaces import IMimetype
from Products.PloneTestCase import PloneTestCase
from Products.OpenXml.config import office_mimetypes
import common

class MTRTestCase(PloneTestCase.PloneTestCase):

    def afterSetUp(self):

        portal = self.getPortal()
        self.mtr = getToolByName(portal, 'mimetypes_registry')


    def testInstallation(self):
        """Checking installation of our Mime types"""

        mtr = self.mtr
        for mt_dict in office_mimetypes:
            mt_string = mt_dict['mimetypes'][0]
            mimetype = mtr.lookup(mt_string)
            self.failUnless(
                len(mimetype) > 0,
                "Didn't find MimeType obj for %s" % mt_string)
            if len(mimetype) > 0:
                self.failUnless(
                    IMimetype.isImplementedBy(mtr.lookup(mt_string)[0]),
                    "Didn't find MimeType obj for %s" % mt_string)
        return


    def testExtensions(self):
        """Finding mimetypes by extension"""

        mtr = self.mtr
        for mt_dict in office_mimetypes:
            ext = mt_dict['extensions'][0]
            expected = mt_dict['mimetypes'][0]
            got = mtr.lookupExtension(ext).normalized()
            self.failUnlessEqual(expected, got)
        return


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(MTRTestCase))
    return suite




#-*- coding: utf-8 -*-
# $Id: install.py 57531 2008-01-23 16:40:27Z glenfant $
"""OpenXml installation"""

from Products.OpenXml.setuphandlers import removeOpenXml

def uninstall(self):
    # Note that there is no GenericSetup support for uninstalling today.

    removeOpenXml(self)
    return



#-*- coding: utf-8 -*-
# $Id: setuphandlers.py 57533 2008-01-23 16:49:16Z glenfant $
"""Setup handlers for OpenXml"""

from Products.CMFCore.utils import getToolByName
import config
from Products.OpenXml import logger

def setupOpenXml(context):
    """Add MS Office 2007 content types to MimetypesRegistry"""

    site = context.getSite()

    # Adding our file types to MTR
    mtr = getToolByName(site, 'mimetypes_registry')
    for mt_dict in config.office_mimetypes:
        main_mt = mt_dict['mimetypes'][0]
        mt_name = mt_dict['name']
        if bool(mtr.lookup(main_mt)):
            # Already installed
            logger.info("%s (%s) Mime type already installed, skipped", main_mt, mt_name)
            continue
        mtr.manage_addMimeType(
            mt_name,
            mt_dict['mimetypes'],
            mt_dict['extensions'],
            mt_dict['icon_path'],
            binary=True,
            globs=mt_dict['globs'])
        logger.info("%s (%s) Mime type installed", main_mt, mt_name)

    # Registering our transform in PT
    transforms_tool = getToolByName(site, 'portal_transforms')
    if config.TRANSFORM_NAME not in transforms_tool.objectIds():
        # Not already installed
        transforms_tool.manage_addTransform(config.TRANSFORM_NAME, 'Products.OpenXml.transform')
    return


def removeOpenXml(context):
    """Removing various resources from plone site"""

    # At the moment, there's no uninstall support in GenericSetup. So
    # this is run by the old style quickinstaller uninstall handler.

    site = context
    # Removing our types from MTR
    mtr = getToolByName(site, 'mimetypes_registry')
    mt_ids = [mt_dict['mimetypes'][0] for mt_dict in config.office_mimetypes]
    mtr.manage_delObjects(mt_ids)

    # Removing our transform from PT
    transforms_tool = getToolByName(site, 'portal_transforms')
    transforms_tool.unregisterTransform(config.TRANSFORM_NAME)
    return





# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

_home_dir = os.path.join(os.path.dirname(__file__), 'Products', 'OpenXml')

_version = file(os.path.join(_home_dir, 'version.txt'), 'r').read().strip()
_long_description = file(os.path.join(_home_dir, 'README.txt')).read()
_long_description += file(os.path.join(_home_dir, 'HISTORY.txt')).read() 

setup(
    name='Products.OpenXml',
    version=_version,
    description="OpenXml documents support for Plone",
    long_description=_long_description,
    classifiers=[
    "Framework :: Plone",
    "Framework :: Zope2",
    "Programming Language :: Python",
    ],
    keywords='Zope CMF Plone openxml',
    author='Gilles Lenfant',
    author_email='gilles.lenfant@gmail.com',
    url='http://svn.plone.org/svn/collective/Products.OpenXml',
    license='GPLv2',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['Products'],
    include_package_data=True,
    zip_safe=False,
    download_url='http://plone.org/products/openxml',
    install_requires=[
    'setuptools',
    'openxmllib',
    'plone.app.controlpanel',
    'plone.app.portlets',
    'plone.portlets',
    'plone.memoize',
    'plone.i18n',
    ],
)



#-*- coding: utf-8 -*-
# $Id: transform.py 67356 2008-06-27 10:01:12Z glenfant $
"""
Our transform (most job is done by openxmllib)
"""
import mimetypes
import openxmllib
from openxmllib import contenttypes as ct
from Products.PortalTransforms.interfaces import itransform
from config import SITE_CHARSET, TRANSFORM_NAME
from Products.OpenXml import logger

class openxml_to_text:
    __implements__ = itransform
    __name__ = TRANSFORM_NAME

    inputs = (
        # Wordprocessing formats
        ct.CT_WORDPROC_DOCX_PUBLIC,
        ct.CT_WORDPROC_DOCM_PUBLIC,
        ct.CT_WORDPROC_DOTX_PUBLIC,
        ct.CT_WORDPROC_DOTM_PUBLIC,

        # Spreadsheet formats
        ct.CT_SPREADSHEET_XLSX_PUBLIC,
        ct.CT_SPREADSHEET_XLSM_PUBLIC,
        ct.CT_SPREADSHEET_XLTX_PUBLIC,
        ct.CT_SPREADSHEET_XLTM_PUBLIC,
        # FIXME: note sure we can honour below types...
#        '*.xlam': ct.CT_SPREADSHEET_XLAM_PUBLIC,
#        '*.xlsb': ct.CT_SPREADSHEET_XLSB_PUBLIC

        # Presentation formats
        ct.CT_PRESENTATION_PPTX_PUBLIC,
        ct.CT_PRESENTATION_PPTM_PUBLIC,
        ct.CT_PRESENTATION_POTX_PUBLIC,
        ct.CT_PRESENTATION_POTM_PUBLIC,
        ct.CT_PRESENTATION_PPSX_PUBLIC,
        ct.CT_PRESENTATION_PPSM_PUBLIC,
        # FIXME: Not sure we can honour below types
#        '*.ppam': ct.CT_PRESENTATION_PPAM_PUBLIC
        )

    output = 'text/plain'

    output_encoding = SITE_CHARSET

    def __init__(self,name=None):
        if name:
            self.__name__=name
        return

    def name(self):
        return self.__name__

    def convert(self, orig, data, **kwargs):

        #orig_file = kwargs.get('filename') or 'unknown.xxx'
        mimetype = kwargs.get('mimetype')
        filename = kwargs.get('filename') or 'unknown.xxx'
        if mimetype is None:
            mimetype = mimetypes.guess_type(filename)[0]
        try:
            doc = openxmllib.openXmlDocument(orig, mimetype)
            data.setData(doc.indexableText().encode(SITE_CHARSET, 'replace'))
        except ValueError, e:
            # Crappy data provided to the transform.
            logger.error("Crappy file provided, returning empty text", exc_info=True)
            data.setData('')
        return data

def register():
    return openxml_to_text()



#-*- coding: utf-8 -*-
# $Id: test_pt.py 57531 2008-01-23 16:40:27Z glenfant $
"""Testing portal_transforms settings"""
import os
import StringIO
from Products.CMFCore.utils import getToolByName
from Products.PloneTestCase import PloneTestCase
from Products.OpenXml.config import TRANSFORM_NAME
import common

class PTTestCase(PloneTestCase.PloneTestCase):

    def afterSetUp(self):

        portal = self.getPortal()
        self.pt = getToolByName(portal, 'portal_transforms')


    def testInstallation(self):
        """Checking installation of our transform"""

        self.failUnless(TRANSFORM_NAME in self.pt.objectIds(spec='Transform'),
                        "%s transform expected" % TRANSFORM_NAME)
        return


    def testATfileSearchableText(self):
        """Do we index the text of an openxml office file"""

        self.loginAsPortalOwner()
        class fakefile(StringIO.StringIO):
            pass
        this_dir = os.path.dirname(os.path.abspath(__file__))
        test_filename = os.path.join(this_dir, 'wordprocessing1.docx')
        fakefile = fakefile(file(test_filename, 'rb').read())
        fakefile.filename = 'wordprocessing1.docx'
        file_id = self.portal.invokeFactory('File', fakefile.filename, file=fakefile)
        file_item = getattr(self.portal, file_id)
        # We sample some words from the file and its metadata
        words = ("The", "subject", "of", "the", "document", "custom_value_1",
                 "Lenfant", "example", "title")
        st = file_item.SearchableText()
        for word in words:
            self.failUnless(word in st, "Extected %s in indexable text" % word)
        return


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(PTTestCase))
    return suite

